;This example shows how to use the function UpdateNormalsAngle.
;The function is used much like the regular UpdateNormals in B3D. Unlike the original function, 
;it doesn't just smooth all edges, but provides an option to set an angle, to which two faces are smoothed.
;If an angle of 180 or above is used, it works exactly like the regular B3D-function. 
;An angle of 0 disables smoothing for the provided mesh, each edge is hard shaped.
;The angles between can be used for an improved shading, where hard edges stay and soft ones appear rounded.
;Just try some values, which angle works best for your mesh.

;For example take a look at that bf109 model provided within the zip file. 
;With the original function or an angle of 180 degree each wing itself and especially the part where it's
;connected to the body looks weird. 
;An angle of 90 or 120 is okay for the shading of the wings, but the wing-to-body-connection isn't fixed yet.
;On the other hand an angle equal or below 30 doesn't smooth the round engine part, so it looks very "Low Poly".
;So a value between 60 and 75 works best for that bf109 mesh.

;30.12.2007 Robert Hierl / www.mein-murks.de
Graphics3D 800,600,16,2


;camera
camera=CreateCamera()
MoveEntity camera, 0, -30, -300

;light
light=CreateLight(1)
TurnEntity light,45,45,0


;mesh
mesh = LoadMesh("bf109.b3d")
If Not mesh Then RuntimeError("File not found")

Angle = 180
UpdateNormalsAngle mesh, Angle

TurnEntity mesh, -25,60,20


timer = CreateTimer(90)
time_old = MilliSecs()
While Not KeyHit(1)
;	time based movement
	time_new = MilliSecs()
	factor#  = (time_new - time_old) / 10.0
	time_old = time_new


	;Arrow Keys / PageUp / PageDown : Turn Mesh	
	TurnEntity mesh, (KeyDown(200)-KeyDown(208))*factor#, (KeyDown(205)-KeyDown(203))*factor#, (KeyDown(209)-KeyDown(201))*factor#


	;Space : Change Visualization of Normals( none, Vertex Normals, Face Normals )
	If KeyHit(57)
		mode = mode + 1
		If mode > 2 Then mode = 0
	EndIf

	
	;NumKey + and - : Change Angle
	addAngle = KeyHit(78) + KeyHit(27)
	subAngle = KeyHit(74) + KeyHit(53)
 
	For i = 1 To addAngle
		Angle = Angle + 15
		If Angle > 90  Then Angle = Angle + 15
		If Angle > 180 Then Angle = 0
	Next

	For i = 1 To subAngle
		Angle = Angle - 15
		If Angle > 90 Then Angle = Angle - 15
		If Angle < 0 Then Angle = 180
	Next

	If addAngle Or subAngle Then UpdateNormalsAngle(mesh, Angle)

	;draw
	RenderWorld
	
	Color 0,255,0
	
	Select mode
		Case 1
			DrawVertexNormals(mesh,camera, 2, .2)
			modename$ = "Showing Vertex Normals"
		Case 2
			DrawFaceNormals(mesh, camera, 2, .2)
			modename$ = "Showing Face Normals"
		Default
			modename$ = "Hiding Normals"
	End Select

	Text 10, 10, "Use Arrow Keys/PageUp/PageDown to rotate model"
	Text 10, 25, "Angle < + / - >:   " + Angle
	Text 10, 40, "View Mode <SPACE>: " + modeName$


	WaitTimer timer
	Flip 0
Wend



;*********************************************************
Function DrawVertexNormals(entity,camera, numSurface=1, divide#=10)
	;Draw 

	Local surface

	surface=GetSurface(entity,numSurface)
	
	For i=0 To CountVertices(surface)-1

		DrawNormal(camera, entity, VertexX#(surface, i), VertexY#(surface, i),VertexZ#(surface, i), VertexNX#(surface, i), VertexNY#(surface, i),VertexNZ#(surface, i), divide)
		
	Next

End Function


Function DrawFaceNormals(entity,camera, numSurface=1,divide#=10)

	Local surface, vx#, vy#, vz#, vnx#, vny#, vnz#, v1, v2, v3

	surface=GetSurface(entity,numSurface)
	
	For triangle = 0 To CountTriangles(surface)-1

		v1 = TriangleVertex(surface, triangle, 0)
		v2 = TriangleVertex(surface, triangle, 1)
		v3 = TriangleVertex(surface, triangle, 2) 
         
		vx# = (VertexX#(surface,v1) + VertexX#(surface,v2) + VertexX#(surface,v3))/3
		vy# = (VertexY#(surface,v1) + VertexY#(surface,v2) + VertexY#(surface,v3))/3
		vz# = (VertexZ#(surface,v1) + VertexZ#(surface,v2) + VertexZ#(surface,v3))/3

		vnx# = (VertexNX#(surface,v1) + VertexNX#(surface,v2) + VertexNX#(surface,v3))/3
		vny# = (VertexNY#(surface,v1) + VertexNY#(surface,v2) + VertexNY#(surface,v3))/3
		vnz# = (VertexNZ#(surface,v1) + VertexNZ#(surface,v2) + VertexNZ#(surface,v3))/3

		DrawNormal(camera, entity, vx#, vy#, vz#, vnx#, vny#, vnz#, divide#)

   Next

End Function


Function DrawNormal(camera, entity, x#, y#, z#, nX#, nY#, nZ#, divide#=1)

	Local startX, startY
	
	;transform vector position to entity
	TFormPoint x#, y#,z#, entity,0
	x# = TFormedX#()
	y# = TFormedY#()
	z# = TFormedZ#()

	;transform vector position to camera(transform to xy-coordinates) 
	CameraProject camera, x#, y#, z#
	
	If ProjectedZ#() = 0 Then Return
	startX = ProjectedX# ()
	startY = ProjectedY# ()

	;transform vector to entity
	TFormPoint nX#, nY#,nZ#, entity,0 
	nX# = TFormedX#() / divide# + x#
	nY# = TFormedY#() / divide# + y#
	nZ# = TFormedZ#() / divide# + z#

	;transform vector to camera
	CameraProject camera, nX#, nY#, nZ#
	If ProjectedZ#() = 0 Then Return
	
	;draw line
	Line startX, startY, ProjectedX#(),ProjectedY#()

End Function



Include "UpdateNormals.bb"